@@ -33,7 +33,7 @@
 #include "file.h"
 
 #ifndef	lint
-FILE_RCSID("@(#)$Id: apprentice.c,v 1.29 1999/10/31 22:23:03 christos Exp $")
+FILE_RCSID("@(#)$Id: apprentice.c,v 1.33 2000/08/05 17:36:47 christos Exp $")
 #endif	/* lint */
 
 #define	EATAB {while (isascii((unsigned char) *l) && \
@@ -42,6 +42,13 @@ FILE_RCSID("@(#)$Id: apprentice.c,v 1.29 1999/10/31 22:23:03 christos Exp $")
 			tolower((unsigned char) (l)) : (l))
 
 
+#ifdef __EMX__
+  char PATHSEP=';';
+#else
+  char PATHSEP=':';
+#endif
+
+
 static int getvalue	__P((struct magic *, char **));
 static int hextoint	__P((int));
 static char *getstr	__P((char *, char *, int, int *));
@@ -54,8 +61,8 @@ static int apprentice_1	__P((const char *, int));
 
 int
 apprentice(fn, check)
-const char *fn;			/* list of magic files */
-int check;			/* non-zero? checking-only run. */
+	const char *fn;			/* list of magic files */
+	int check;			/* non-zero? checking-only run. */
 {
 	char *p, *mfn;
 	int file_err, errs = -1;
@@ -73,7 +80,7 @@ int check;			/* non-zero? checking-only run. */
 	fn = strcpy(mfn, fn);
   
 	while (fn) {
-		p = strchr(fn, ':');
+		p = strchr(fn, PATHSEP);
 		if (p)
 			*p++ = '\0';
 		file_err = apprentice_1(fn, check);
@@ -93,8 +100,8 @@ int check;			/* non-zero? checking-only run. */
 
 static int
 apprentice_1(fn, check)
-const char *fn;			/* name of magic file */
-int check;			/* non-zero? checking-only run. */
+	const char *fn;			/* name of magic file */
+	int check;			/* non-zero? checking-only run. */
 {
 	static const char hdr[] =
 		"cont\toffset\ttype\topcode\tmask\tvalue\tdesc";
@@ -106,8 +113,8 @@ int check;			/* non-zero? checking-only run. */
 	if (f==NULL) {
 		if (errno != ENOENT)
 			(void) fprintf(stderr,
-			"%s: can't read magic file %s (%s)\n", 
-			progname, fn, strerror(errno));
+			    "%s: can't read magic file %s (%s)\n", 
+			    progname, fn, strerror(errno));
 		return -1;
 	}
 
@@ -134,8 +141,8 @@ int check;			/* non-zero? checking-only run. */
  */
 uint32
 signextend(m, v)
-struct magic *m;
-uint32 v;
+	struct magic *m;
+	uint32 v;
 {
 	if (!(m->flag & UNSIGNED))
 		switch(m->type) {
@@ -175,8 +182,8 @@ uint32 v;
  */
 static int
 parse(l, ndx, check)
-char *l;
-int *ndx, check;
+	char *l;
+	int *ndx, check;
 {
 	int i = 0, nd = *ndx;
 	struct magic *m;
@@ -184,17 +191,20 @@ int *ndx, check;
 
 #define ALLOC_INCR	200
 	if (nd+1 >= maxmagic){
-	    maxmagic += ALLOC_INCR;
-	    if ((magic = (struct magic *) realloc(magic, 
-						  sizeof(struct magic) * 
+		maxmagic += ALLOC_INCR;
+		if ((m = (struct magic *) realloc(magic, sizeof(struct magic) * 
 						  maxmagic)) == NULL) {
-		(void) fprintf(stderr, "%s: Out of memory.\n", progname);
-		if (check)
-			return -1;
-		else
-			exit(1);
-	    }
-	    memset(&magic[*ndx], 0, sizeof(struct magic) * ALLOC_INCR);
+			(void) fprintf(stderr, "%s: Out of memory.\n",
+			    progname);
+			if (magic)
+				free(magic);
+			if (check)
+				return -1;
+			else
+				exit(1);
+		}
+		magic = m;
+		memset(&magic[*ndx], 0, sizeof(struct magic) * ALLOC_INCR);
 	}
 	m = &magic[*ndx];
 	m->flag = 0;
@@ -336,6 +346,28 @@ int *ndx, check;
 		++l;
 		m->mask = signextend(m, strtoul(l, &l, 0));
 		eatsize(&l);
+	} else if (STRING == m->type) {
+		m->mask = 0L;
+		if (*l == '/') { 
+			while (!isspace(*++l)) {
+				switch (*l) {
+				case CHAR_IGNORE_LOWERCASE:
+					m->mask |= STRING_IGNORE_LOWERCASE;
+					break;
+				case CHAR_COMPACT_BLANK:
+					m->mask |= STRING_COMPACT_BLANK;
+					break;
+				case CHAR_COMPACT_OPTIONAL_BLANK:
+					m->mask |=
+					    STRING_COMPACT_OPTIONAL_BLANK;
+					break;
+				default:
+					magwarn("string extension %c invalid",
+					    *l);
+					return -1;
+				}
+			}
+		}
 	} else
 		m->mask = ~0L;
 	EATAB;
@@ -412,8 +444,8 @@ GetDesc:
  */
 static int
 getvalue(m, p)
-struct magic *m;
-char **p;
+	struct magic *m;
+	char **p;
 {
 	int slen;
 
@@ -436,14 +468,14 @@ char **p;
  */
 static char *
 getstr(s, p, plen, slen)
-register char	*s;
-register char	*p;
-int	plen, *slen;
+	char	*s;
+	char	*p;
+	int	plen, *slen;
 {
 	char	*origs = s, *origp = p;
 	char	*pmax = p + plen - 1;
-	register int	c;
-	register int	val;
+	int	c;
+	int	val;
 
 	while ((c = *s++) != '\0') {
 		if (isspace((unsigned char) c))
@@ -539,13 +571,17 @@ out:
 /* Single hex char to int; -1 if not a hex char. */
 static int
 hextoint(c)
-int c;
+	int c;
 {
-	if (!isascii((unsigned char) c))	return -1;
-	if (isdigit((unsigned char) c))		return c - '0';
-	if ((c>='a')&&(c<='f'))	return c + 10 - 'a';
-	if ((c>='A')&&(c<='F'))	return c + 10 - 'A';
-				return -1;
+	if (!isascii((unsigned char) c))
+		return -1;
+	if (isdigit((unsigned char) c))
+		return c - '0';
+	if ((c >= 'a')&&(c <= 'f'))
+		return c + 10 - 'a';
+	if (( c>= 'A')&&(c <= 'F'))
+		return c + 10 - 'A';
+	return -1;
 }
 
 
@@ -554,11 +590,11 @@ int c;
  */
 void
 showstr(fp, s, len)
-FILE *fp;
-const char *s;
-int len;
+	FILE *fp;
+	const char *s;
+	int len;
 {
-	register char	c;
+	char	c;
 
 	for (;;) {
 		c = *s++;
@@ -613,7 +649,7 @@ int len;
  */
 static void
 eatsize(p)
-char **p;
+	char **p;
 {
 	char *l = *p;
 
